@@ -5,20 +5,15 @@ import (
 	"fmt"
 	"testing"
 
-	"github.com/spf13/pflag"
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
 )
 
-// Interface which flags must satisfy - only defined for _test.go
-// since we don't want to pull in pflag here
-type flagger interface {
-	pflag.Value
-	json.Unmarshaler
-}
-
-// Check it satisfies the interface
-var _ flagger = (*SizeSuffix)(nil)
+// Check it satisfies the interfaces
+var (
+	_ Flagger   = (*SizeSuffix)(nil)
+	_ FlaggerNP = SizeSuffix(0)
+)
 
 func TestSizeSuffixString(t *testing.T) {
 	for _, test := range []struct {
@@ -41,7 +36,7 @@ func TestSizeSuffixString(t *testing.T) {
 	}
 }
 
-func TestSizeSuffixByteShortUnit(t *testing.T) {
+func TestSizeSuffixByteUnit(t *testing.T) {
 	for _, test := range []struct {
 		in   float64
 		want string
@@ -58,30 +53,6 @@ func TestSizeSuffixByteShortUnit(t *testing.T) {
 		{1 * 1024 * 1024 * 1024 * 1024 * 1024 * 1024, "1 EiB"},
 		{-1, "off"},
 		{-100, "off"},
-	} {
-		ss := SizeSuffix(test.in)
-		got := ss.ByteShortUnit()
-		assert.Equal(t, test.want, got)
-	}
-}
-
-func TestSizeSuffixByteUnit(t *testing.T) {
-	for _, test := range []struct {
-		in   float64
-		want string
-	}{
-		{0, "0 Byte"},
-		{102, "102 Byte"},
-		{1024, "1 KiByte"},
-		{1024 * 1024, "1 MiByte"},
-		{1024 * 1024 * 1024, "1 GiByte"},
-		{10 * 1024 * 1024 * 1024, "10 GiByte"},
-		{10.1 * 1024 * 1024 * 1024, "10.100 GiByte"},
-		{10 * 1024 * 1024 * 1024 * 1024, "10 TiByte"},
-		{10 * 1024 * 1024 * 1024 * 1024 * 1024, "10 PiByte"},
-		{1 * 1024 * 1024 * 1024 * 1024 * 1024 * 1024, "1 EiByte"},
-		{-1, "off"},
-		{-100, "off"},
 	} {
 		ss := SizeSuffix(test.in)
 		got := ss.ByteUnit()
